// Class: IterationController
//
// Author: Alyce Brady
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

import java.awt.Color;

import java.lang.reflect.Constructor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import edu.kzoo.grid.BoundedGrid;
import edu.kzoo.grid.ColorBlock;
import edu.kzoo.grid.Grid;
import edu.kzoo.grid.Location;
import edu.kzoo.grid.display.ScrollableGridDisplay;
import edu.kzoo.grid.gui.ActiveGridAppController;

/**
 *  Grid Iterator Lab:<br>
 *
 *    The <code>IterationController</code> class provides the methods
 *    for stepping through a grid using one of a number of
 *    iterators (which must be registered using the <code>register</code>
 *    method).  It places colored blocks in the cells of the grid
 *    as it goes to illustrate the traversal.
 *
 *  @author Alyce Brady
 *  @version 1 September 2004
 **/
public class IterationController extends ActiveGridAppController
{
    // iterator constructor parameter types (GridIterator and subclasses)
    private static final Class[] IT_CONST_PARAM_TYPES = {BoundedGrid.class};

    // Specify traversal iterators; algNameList is a list of iterator
    // names in the order in which they are registered, while iteratorMap
    // maps those names to GridIterator or one of its subclasses.
    private static List algNameList = new ArrayList();
    private static Map iteratorMap = new HashMap();

    // Instance Variables: Encapsulated data for EACH iteration controller
    private Grid grid;                  // grid through which to step
    private ScrollableGridDisplay disp; // object to display grid
    private int numLocationsInGrid;     // number of locations in grid
    private GridIterator iter;          // iterator that steps through grid
    private int numLocationsVisited;    // number of locs visited in traversal
    private Color highlightColor;       // color to highlight visited cells
    private boolean readyToStop;        // has the iteration finished?


  // Class Methods to Register and Find Iterators

    /** Registers the specified class with a name.
     *      @param name  name (button label) of traversal or drawing algorithm
     *      @param iteratorClass class that implements the associated iterator
     **/
    public static void register(String name, Class iteratorClass)
    {
        if ( ! GridIterator.class.isAssignableFrom(iteratorClass) )
            throw new IllegalArgumentException("iterator " + 
              iteratorClass.getName() + " must be a subclass of GridIterator");
        algNameList.add(name);
        iteratorMap.put(name, iteratorClass);
    }

    /** Gets the list of registered algorithm names.
     *      @return  name (button label) of traversal or drawing algorithm
     **/
    public static List getAlgNames()
    {
        return algNameList;
    }

    /** Constructs the iterator with the specified name.
     *      @param name  name of traversal or drawing algorithm
     *      @return      iterator that traverses in the specified order
     **/
    public static GridIterator getIterator(String name, BoundedGrid grid)
    {
        Class itClass = (Class) iteratorMap.get(name);
        try
        {
            Class[] pTypes = new Class[] {BoundedGrid.class};
            Constructor itCons = itClass.getConstructor(pTypes);
            Object[] params = new Object[] {grid};
            return (GridIterator) itCons.newInstance(params);
        }
        catch (Exception e)
        {  throw new IllegalArgumentException("Error constructing " +
                                              itClass + " iterator: " + e);
        }
    }

    /** Resets the controller state to prepare for a traversal or iteration
     *  through the specified grid using the iterator with the given name.
     *      @param name  name of the iteration technique to use
     *      @param grid  grid through which to iterate
     *      @param color color to use to mark the traversal
     **/
    public void prepareForIteration(String name, BoundedGrid grid, Color color)
    {
        this.grid = grid;
        this.grid.removeAll();
        this.numLocationsInGrid = grid.numRows() * grid.numCols();
        this.numLocationsVisited = 0;
        this.highlightColor = color;
        this.readyToStop = false;
        this.iter = getIterator(name, grid);
    }
    
    public void step()
    {
        // Use the appropriate iterator to traverse through the grid,
        // placing color blocks as we go to animate the traversal algorithm.
        if ( iter.hasNext() )
        {
            Location nextLoc = (Location)iter.next();
            if ( grid.isEmpty(nextLoc) )
                new ColorBlock(highlightColor, grid, nextLoc);
            else
                // Be informative for pedagogical reasons.
                System.out.println("WARNING - TRAVERSAL BUG: " +
                    "revisiting location " + nextLoc);
        }
        else
            readyToStop = true;

        numLocationsVisited++;
    }

    /** Determines whether the iteration has reached
     *  a desired stopping state.  Will return <code>true</code> if
     *  the traversal has finished or if it appears to be in an
     *  unproductive loop (it has visited more locations than exist
     *  in the grid). 
     *    @return <code>true</code> if the application should stop 
     **/
    public boolean hasReachedStoppingState()
    {
        if ( ! readyToStop && numLocationsVisited > numLocationsInGrid )
        {
            // Be informative for pedagogical reasons.
            System.out.print("WARNING - too many iterations;");
            System.out.println(" stopping traversal.");
            readyToStop = true;
        }
        return readyToStop;
    }

}
