// Class: IteratorGUI
//
// Author: Alyce Brady
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Iterator;

import edu.kzoo.grid.BoundedGrid;
import edu.kzoo.grid.Location;
import edu.kzoo.grid.display.ColorBlockDisplay;
import edu.kzoo.grid.display.DisplayMap;
import edu.kzoo.grid.gui.ControlButton;
import edu.kzoo.grid.gui.EnabledDisabledStates;
import edu.kzoo.grid.gui.SteppedGridAppFrame;
import edu.kzoo.grid.gui.nuggets.BGColorChoiceMenu;
import edu.kzoo.grid.gui.nuggets.BasicHelpMenu;
import edu.kzoo.grid.gui.nuggets.ClearGridButton;
import edu.kzoo.grid.gui.nuggets.ColorChoiceMenu;
import edu.kzoo.grid.gui.nuggets.MinimalFileMenu;
import edu.kzoo.grid.gui.nuggets.NewBoundedGridButton;

/**
 *  Grid Iterator Lab:<br>
 *
 *    The <code>IteratorGUI</code> class provides the graphical user
 *    interface for a program that draws in a grid by placing
 *    color blocks in its cells.
 *
 *  @author Alyce Brady (based on code by Julie Zelenski)
 *  @version 1 September 2004
 **/
public class IteratorGUI extends SteppedGridAppFrame
{
  // Constants: shared among all objects of the class

    // Define some common graphical user interface attributes.
    protected static final int MIN_CELL_SIZE = 10;
    protected static final boolean DISPLAY_GRID_AFTER_BUTTON_PRESSES = true;

  // Instance Variables: Encapsulated data for EACH IteratorGUI object
    protected ColorChoiceMenu drawingColorMenu;


  // constructor

    /** Constructs the graphical user interface for the Iterator
     *  traversal program.
     *    @param title frame title
     *    @param viewingWidth  the width of the viewing area
     *    @param viewingHeight the height of the viewing area
     *    @param maxDelayMsecs longest delay value for slider, in milliseconds
     *    @param minDelayMsecs shortest delay value for slider, in milliseconds
     **/
    public IteratorGUI(String title, int viewingWidth, int viewingHeight,
                         int maxDelayMSecs, int minDelayMsecs)
    {
        // Construct and initialize the GUI window.
        super(new IterationController(), DISPLAY_GRID_AFTER_BUTTON_PRESSES);

        includeMenu(new MinimalFileMenu());
        includeMenu(new BasicHelpMenu("GridIterator", "Your Name Here",
            "with assistance from (whom?)",
            "version date", "file:GridIteratorHelp.html"));

        // Include a New Grid button.
        includeControlComponent(new NewBoundedGridButton(this,
                                                         "Create New Grid"),
                                EnabledDisabledStates.NEEDS_APP_WAITING);

        // Include a background color chooser component.
        ColorChoiceMenu bgColorChooser = new BGColorChoiceMenu(this);
        includeControlComponent(bgColorChooser,
                               EnabledDisabledStates.NEEDS_APP_WAITING);

        // Include a color chooser component for color blocks.
        drawingColorMenu = new ColorChoiceMenu("Fill Color:");
        includeControlComponent(drawingColorMenu,
                                EnabledDisabledStates.NEEDS_APP_WAITING);

        // Generate control buttons from the list of algorithms and include
        // them in the user interface.  The
        // target object needs to know when a new grid is created, so register
        // it as a grid change listener.
        includeControlComponents(createTraversalAlgButtons(),
                            EnabledDisabledStates.NEEDS_GRID_AND_APP_WAITING);

        // Include a stop button to stop run-away traversals.
        includeStopButton(DISPLAY_GRID_AFTER_BUTTON_PRESSES);

        // Include a clear button to clear out all color blocks.
        includeControlComponent(new ClearGridButton(this, "Clear",
                                            DISPLAY_GRID_AFTER_BUTTON_PRESSES),
                            EnabledDisabledStates.NEEDS_GRID_AND_APP_WAITING);

        // Include a speed slider.
        includeSpeedSlider(maxDelayMSecs, minDelayMsecs);

        // Construct the visible window with the specified components.
        constructWindowContents(title, bgColorChooser.currentColor(),
                                viewingWidth, viewingHeight,
                                MIN_CELL_SIZE);

        // Specify what object knows how to display color blocks.
        DisplayMap.associate("edu.kzoo.grid.ColorBlock",
                             new ColorBlockDisplay());
    }

    /** Defines contents of main window panel.  Redefine this method
     *  to define panels other than a grid display panel, control panel,
     *  or speed slider bar panel, or to alter the layout or border of
     *  the main contents panel.
     *   @return the panel that will serve as the window's content panel
     **/
    protected JPanel defineContent()
    {
        JPanel content = new JPanel();
        content.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
        content.setLayout(new BorderLayout());

        // Create a panel to display the grid and speed slider bar
        // and another for the control buttons (e.g., a Reset button),
        // and add them to the main panel. 
        JPanel gridAndSliderPanel = new JPanel();
        gridAndSliderPanel.setLayout(new BorderLayout());

        gridAndSliderPanel.add(makeDisplayPanel(), BorderLayout.CENTER);

        JComponent sliderPanel = makeSliderPanel();
        if ( sliderPanel != null )
            gridAndSliderPanel.add(sliderPanel, BorderLayout.SOUTH);

        content.add(gridAndSliderPanel, BorderLayout.CENTER);

        JComponent controlPanel = makeControlPanel(null);
        if ( controlPanel != null )
            content.add(controlPanel, BorderLayout.WEST);

        return content;
    }

    /** Shows the contents of a single location in the grid.  This allows
     *  for speedier display than the showGrid method because it only updates
     *  the contents of the one, specified location.
     *  (Precondition: must have called <code>setGrid</code>.)
     **/
    public void showLocation(Location loc)
    {
        getDisplay().updateLocation(loc);
        if ( getDelay() > 0 )
        {
            try 
            {
              Thread.sleep(getDelay());
            }
            catch (InterruptedException e) {}
        }
    }

    /** Creates a list of the traversal algorithm buttons.
     **/
    protected ArrayList createTraversalAlgButtons()
    {
        ArrayList theList = new ArrayList();

        // For each algorithm in algNameList, make a threaded control button.
        Iterator iter = IterationController.getAlgNames().iterator();
        while ( iter.hasNext() )
        {
            // Make a button that runs this traversal algorithm.
            String name = (String) iter.next();
            JButton algButton =
                new ControlButton(this, name, DISPLAY_GRID_AFTER_BUTTON_PRESSES)
                {  public void act()
                    { runTraversal(getText()); }
                };
            theList.add(algButton);
        }

        return theList;
    }

    /** Animates a traversal algorithm.  It should be easy to visually
     *  verify a correct traversal algorithm.
     *      @param  buttonLabel   name (label) of traversal algorithm
     **/
    public void runTraversal(String buttonLabel)
    {
        // Initialize the iteration controller to specify which traversal
        // strategy to run and what color to use to show the traversal.
        IterationController controller = (IterationController) getController();
        controller.prepareForIteration(buttonLabel, (BoundedGrid)getGrid(),
                                  drawingColorMenu.currentColor());
        // Show the initial configuration.
        showGrid();

        // Run the traversal.
        run();
    }

}
